<?php
Namespace App\Controller\Admin\Inventory;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use App\Model\Admin\HomegearGroup;
use App\Model\Admin\HomegearDevice;

class GroupController extends Controller
{
    public function indexAction(Request $request, HomegearGroup $group)
    {
        return $this->render('Admin/Inventory/Group/list.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'groups' => $group->loadAll()
        ));
    }

    public function editAction($id, Request $request, HomegearGroup $group, HomegearDevice $device)
    {
        $expert = false;
        $userSettings = \Homegear\Homegear::getUserMetadata($this->getUser()->getUsername());
        if (array_key_exists('expert',$userSettings) && $userSettings['expert'] == 'true') {
            $expert = true;
        }
        // get the current group
        $currentGroup = $group->loadById($id);
        // get all known devices
        $devices = $device->loadAll();
        if (array_key_exists('devices',$currentGroup) && $currentGroup['devices'] !== NULL) {
            foreach($currentGroup['devices'] as $k => $v) {
                // find device-data for devices in current room
                if ($v['ID'] == 'systemVariable') { continue; }
                $key = array_search(intval($v['ID']), array_column($devices, 'ID'));
                if (isset($key) && intval($key) !== NULL) {
                    $currentGroup['devices'][$k]['name'] = $devices[$key]['name'];
                    if ($v['channel'] == -1) {
                        // only mark device as 'inGroup' if the whole device was added (channel == -1)
                        // so you are still able to add a complete device even if a single channel of this device was added before
                        $devices[$key]['inGroup'] = true;
                    }
                }
            }
        }
        return $this->render('Admin/Inventory/Group/edit.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'group' => $currentGroup,
            'allDevices' => $devices,
            'expert' => $expert
        ));
    }

    public function updateAction($id, Request $request, HomegearGroup $group)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'name' => new Assert\NotBlank,
            'goback' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $group->update($id,$params['name']);
        }
        // TODO: check result and redirect with flash
        if (array_key_exists('goback',$params) && $params['goback'] != '0') {
            return $this->redirectToRoute($params['goback']);
        } else {
            return $this->redirectToRoute('admin.inventory.groups.edit', array('id' => $id));
        }
    }

    public function createAction(Request $request, HomegearGroup $group)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'name' => new Assert\NotBlank,
            'lang' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $group->create($params['name'],$params['lang']);
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.groups');
    }

    public function deleteAction($id, Request $request, HomegearGroup $group)
    {
        $group->delete($id);
        return $this->redirectToRoute('admin.inventory.groups');
    }

    public function updateSortAction($id, Request $request, HomegearGroup $group)
    {
        $params = $request->request->all();
        $group->updateSort($id,$params['position']);
        return new JsonResponse(array('success' => 1));
    }

    public function addDeviceAction($id, Request $request, HomegearGroup $group)
    {
        $params = $request->request->all();
        $group->addDevice($id,$params['device']);
        return new JsonResponse(array('success' => 1));
    }

    public function removeDeviceAction($id, Request $request, HomegearGroup $group)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'deviceid' => new Assert\NotBlank,
            'channel' => new Assert\NotNull,
            'variable' => new Assert\NotNull,
        ));
        $params = $request->query->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $group->removeDevice($id,$params['deviceid'],$params['channel'],$params['variable']);
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.groups.edit', array('id' => $id));
    }

    public function updateDeviceSortAction($id, Request $request, HomegearGroup $group)
    {
        $params = $request->request->all();
        $group->updateDeviceSort($id,$params['deviceId'],$params['position'],$params['channel'],$params['variable']);
        return new JsonResponse(array('success' => 1));
    }

    public function editMetadataAction($id, Request $request, HomegearGroup $group)
    {
        return $this->render('Admin/Inventory/Group/edit_metadata.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'group' => $group->loadMetadata($id)
        ));
    }

    public function addMetadataAction($id, Request $request, HomegearGroup $group)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'key' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            if (!array_key_exists('value',$params)) {
                $params['value'] = '';
            }
            $result = $group->setMetadata($id,$params['key'],$params['value']);
        }
        return $this->redirectToRoute('admin.inventory.groups.edit.metadata',array('id' => $id),302);
    }

    public function deleteMetadataAction($id, Request $request, HomegearGroup $group)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'key' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $group->deleteMetadata($id,$params['key']);
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.groups.edit.metadata',array('id' => $id),301);
    }

    public function updateMetadataAction($id, Request $request, HomegearGroup $group)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'key' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            if (!array_key_exists('value',$params)) {
                $params['value'] = '';
            }
            $result = $group->setMetadata($id,$params['key'],$params['value']);
        }
        return $this->redirectToRoute('admin.inventory.groups.edit.metadata',array('id' => $id),302);
    }

}