<?php
Namespace App\Controller\Admin\Inventory;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use App\Model\Admin\HomegearProfile;
use App\Model\Admin\HomegearRoom;
use App\Model\Admin\HomegearStory;
use App\Model\Admin\HomegearDevice;
use App\Model\Admin\HomegearRole;
use App\Model\Admin\HomegearVariable;
use App\Utils\Utils;

class ProfileController extends Controller
{
    public function indexAction(Request $request, HomegearProfile $profile, HomegearRoom $room, HomegearStory $story)
    {
        $rooms = $room->loadAll($story->loadAll(true));
        $profiles = $profile->loadAll();
        $profiles = (is_null($profiles) ? [] : $profiles);
        foreach ($profiles as $key => $single) {
            if (array_key_exists('room', $single) && $single['room'] != '') {
                $rkey = array_search($single['room'], array_column($rooms, 'ID'));
                if ($rkey !== false) {
                    $profiles[$key]['room'] = $rooms[$rkey];
                }
            }
        }
        return $this->render('Admin/Inventory/Profiles/list.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'profiles' => $profiles
        ));
    }

    public function createAction(Request $request, HomegearProfile $profile)
    {
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'name' => new Assert\NotBlank,
            'lang' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $id = $profile->create($params['name'],$params['lang']);
        }
        return $this->redirectToRoute('admin.inventory.profiles.edit',array('id' => $id),301);
    }

    public function editAction($id, Request $request, HomegearProfile $profile, HomegearRoom $room, HomegearDevice $device, HomegearRole $role, HomegearVariable $variable, HomegearStory $story, Utils $utils)
    {
        $release = $utils->checkRelease();
        $devices = $device->loadAll();
        $roles = $role->loadAll();
        $variables = $variable->loadAll();
        $rooms = $room->loadAll($story->loadAll(true));
        $currentProfile = $profile->loadById($id);
        if (array_key_exists('values', $currentProfile)) {
            foreach ($currentProfile['values'] as $k => $value) {
                if ($value['peerId'] == 0 && $value['channel'] == -1) {
                    $key = array_search($value['variable'], array_column($variables, 'name'));
                    if ($key !== false) {
                        $currentProfile['values'][$k]['url'] = $variables[$key]['url'];
                    }
                } else {
                    $key = array_search($value['peerId'], array_column($devices, 'ID'));
                    if ($key !== false) {
                        if (array_key_exists('name',$devices[$key]) && $devices[$key]['name'] != '') {
                            $currentProfile['values'][$k]['name'] = $devices[$key]['name'];
                        } else {
                            $currentProfile['values'][$k]['name'] = $devices[$key]['address'];
                        }
                    }
                }
            }
        }
        if (array_key_exists('roles', $currentProfile)) {
            foreach ($currentProfile['roles'] as $k => $value) {
                $key = array_search($value['role'], array_column($roles, 'ID'));
                if ($key) {
                    if (array_key_exists('TRANSLATIONS',$roles[$key])) {
                        $currentProfile['roles'][$k]['TRANSLATIONS'] = $roles[$key]['TRANSLATIONS'];
                    }
                }
            }
        }
        return $this->render('Admin/Inventory/Profiles/edit.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'profile' => $currentProfile,
            'rooms' => $rooms,
            'allDevices' => $devices,
            'allVariables' => $variables,
            'allRoles' => $roles,
            'release' => $release
        ));
    }

    public function updateAction($id, Request $request, HomegearProfile $profile)
    {
        $params = $request->request->all();
        $result = $profile->update($id, $params);
        if (array_key_exists('goback', $params) && $params['goback'] != '0') {
            return $this->redirectToRoute($params['goback'],array(),301);
        }
        return $this->redirectToRoute('admin.inventory.profiles.edit',array('id' => $id),301);
    }

    public function deleteAction($id, Request $request, HomegearProfile $profile)
    {
        $profile->delete($id);
        return $this->redirectToRoute('admin.inventory.profiles',array(),301);
    }

    public function getDevice($id, Request $request, HomegearDevice $device)
    {
        $device = $device->loadVariables($id);
        return $this->render('Admin/Inventory/Profiles/device.html.twig', array(
            'device' => $device
        ));
    }

    public function addItemAction($id, Request $request, HomegearDevice $device, HomegearProfile $profile, HomegearVariable $variable)
    {
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array (
                'id' => new Assert\NotBlank,
                'item' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            if ($params['item'] == 'variable') {
                $params['device'] = ($params['id'] != 0 ? $device->loadVariables($params['id']) : $variable->loadByName($params['variable']));
            }
            $result = $profile->addItem($id,$params);
        }
        return $this->redirectToRoute('admin.inventory.profiles.edit',array('id' => $id),301);
    }

    public function updateItemAction($id, Request $request, HomegearDevice $device, HomegearProfile $profile, HomegearVariable $variable)
    {
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array (
                'id' => new Assert\NotBlank,
                'item' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            if ($params['item'] == 'variable') {
                $params['device'] = ($params['id'] != 0 ? $device->loadVariables($params['id']) : $variable->loadByName($params['variable']));
            }
            $result = $profile->updateItem($id,$params);
        }
        return new JsonResponse(['status' => 'ok']);
    }

    public function removeItemAction($id, Request $request, HomegearProfile $profile)
    {
        $params = $request->request->all();
        $profile->removeItem($id, $params);
        return new JsonResponse(['status' => 'ok']);
    }
}