<?php
Namespace App\Model;

/**
 * Provides base functions for Homegear variables
 *
 */
class HomegearVariable
{
    public $hg;

    public function __construct()
    {
        $this->hg = new \Homegear\Homegear;
    }

    /**
     * Load all system variables
     * 
     * @return array()
     */
    public function loadAll($frontend = null) {
        $variables = array();
        $allVars = $this->hg->getAllSystemVariables(true);
        foreach ($allVars as $k => $v) {
            list($type,$value) = $this->_setType($v['VALUE']);
            if ($frontend !== null && (array_key_exists('FLAGS', $v) && (($v['FLAGS'] & 2) || ($v['FLAGS'] & 4)))) {
                // sysvar is hidden for frontend
                continue;
            }
            if ($k == 'homegearStartTime' || $k == 'socketPath') {
                // don't display internal variables
                continue;
            }
            $variables[] = ['name' => $k, 'url' => base64_encode($k), 'type' => $type, 'value' => $value, 'flag' => (array_key_exists('FLAGS', $v) ? $v['FLAGS'] : 0)];
        }
        return (count($variables) > 0 ? $variables : null);
    }

    /**
     * Get the current type of a system variable
     * 
     * @return array()
     */
    public function _setType($v = null)
    {
        $type = 'struct';
        $value = print_r($v, true);
        if (is_string($v)) { $type = 'string'; $value = $v; }
        if (is_int($v)) { $type = 'int'; $value = intval($v); }
        if (is_float($v)) { $type = 'float'; $value = floatval($v);}
        if (is_bool($v)) { $type = 'bool'; $value = boolval($v);}
        if (is_array($v)) { $type = 'array'; $value = print_r($v, true);}
        return [$type, $value];
    }

}